#include "stdafx.h"
#include "MarkerDetector.h"



// G[`FbN
int RegisterPattern( const vector<PATTERN_DATA> &pattern_list )
{
	int n = pattern_list.size();
	for ( int i=0; i<n; i++) {
		arLoadPatt( pattern_list[i].filename.c_str() );
		if ( !(pattern_list[i].enable) ) arDeactivatePatt(i);
	}
	return n;
}


int LoadPatternList( const char *filename, vector<PATTERN_DATA> &pattList )
{	
	ifstream fin(filename);
	if (!fin) {
		cout << filename << " can not open.\n";
		return -1;
	}

	pattList.clear();

	string word;
	string enable;
	PATTERN_DATA patt;

	while (fin >> word) {
		if (word=="[Pattern]") {
			fin >> patt.name >> patt.filename >> patt.width	>> enable;
			patt.enable = (enable=="use" ? true : false);
			patt.id = pattList.size();
			pattList.push_back(patt);
		}
	}	
	fin.close();

	return 0;
}

static void GetDataFromARMarkerInfo3D( ARMarkerInfo* marker, AR_OBJECT &object)
{
	object.area = marker->area;
	object.id = marker->id;
	object.cf = marker->cf;
	object.px = marker->pos[0];
	object.py = marker->pos[1];

	// order correction
	// LeftTop -> RightTop -> RightBottom -> LeftBottom
	for (int j=0; j<4; j++) {
		int c = (j + 4 - marker->dir)%4;
		object.vertex[j][0] = marker->vertex[c][0];
		object.vertex[j][1] = marker->vertex[c][1];
	}

	// normalized vector
	double vec[2];
	vec[0] = ((object.vertex[0][0] - object.px ) + (object.vertex[1][0] - object.px))/2.0;
	vec[1] = ((object.vertex[0][1] - object.py ) + (object.vertex[1][1] - object.py))/2.0;

	double norm = sqrt( vec[0]*vec[0] + vec[1]*vec[1] );
	object.dx = vec[0]/norm;
	object.dy = vec[1]/norm;

	// Get 3D information
	double center[2] = {0.0,0.0};
	double trans[3][4];
	arGetTransMat( marker, center, object.width, trans );
	object.X = trans[0][3];
	object.Y = trans[1][3];
	object.Z = trans[2][3];
	argConvGlpara( trans, object.gltrans );
}


int DetectMarker( ARUint8 *image, int thresh, const vector<PATTERN_DATA> &patt_list, vector<AR_OBJECT> &objects )
{
	ARMarkerInfo	*marker_info;
	int				marker_num;
	double			center[2] = {0.0, 0.0};

	if ( image== NULL) return -1;

	//objects.clear();
	AR_OBJECT one_object;

	if ( arDetectMarkerLite( image, thresh, &marker_info, &marker_num ) ) {
		// use arDetectMarker"Lite"!!
		printf("error in arDetectMaker\n");
		return -1;
	}

	// multi same markers can be exist in the image.
	for (int i=0; i<marker_num; i++) {
		int k = -1;
		for (unsigned int j=0; j<patt_list.size(); j++) {
			if ( marker_info[i].id == patt_list[j].id ) {
				k = j;
				break;
			}
		}
		if (k!=-1 && marker_info[i].cf>=0.6 ) {
			one_object.width = patt_list[k].width;
			one_object.name  = patt_list[k].name;
			GetDataFromARMarkerInfo3D( &marker_info[i], one_object );
			objects.push_back( one_object );
		}
	}

	return 0;
}





/*
static void Begin2d()
{
	// r[|[g̎擾
	GLint viewport[4];
	glGetIntegerv( GL_VIEWPORT, viewport );

	// ݂̎ˉesۑ
	glMatrixMode( GL_PROJECTION );
	glPushMatrix();

	// ˉeɂ
	glLoadIdentity();
	glOrtho( 0.0f, viewport[2], viewport[3], 0.0f, 0.0f, 1.0f);

	// ݂̃fr[sۑ
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();

	// fr[s񏉊
	glLoadIdentity();
}

static void End2d()
{
	// ۑĂȐ𕜌
	glMatrixMode( GL_MODELVIEW );
	glPopMatrix();
	glMatrixMode( GL_PROJECTION );
	glPopMatrix();
}


static void glColor3uc( unsigned char r, unsigned char g, unsigned char b )
{
	glColor3f( (GLfloat)(r/255.0), (GLfloat)(g/255.0), (GLfloat)(b/255.0) );
}


// ̕`i㌴_j
// オ_
static void GLDrawString( int px, int py, const char *string )
{
	void	*font = GLUT_BITMAP_HELVETICA_18;
	GLfloat	offset_y = 18;	// tHgɉāD

	Begin2d();
		glLoadIdentity();
		glTranslatef( px, py + offset_y, 0.0 );
		glRasterPos2i( 0.0, 0.0 );

		for ( int i = 0; i < (int)strlen(string); i++ ) {
			if (string[i] != '\n' ) {
				glutBitmapCharacter( font, string[i] );
			} else {
				glTranslatef( 0.0, 18, 0.0 );
				glRasterPos2i( 0.0, 0.0 );
			}
		}
	End2d();

	return;
}

void ShowAnnotation( AR_OBJECT &object )
{
	// 2`惂[h
	Begin2d();

		glPointSize(5.0);
		glLineWidth(2.0);

		glBegin(GL_LINE_LOOP);
			glVertex2f( object.vertex[0][0], object.vertex[0][1] );
			glVertex2f( object.vertex[1][0], object.vertex[1][1] );
			glVertex2f( object.vertex[2][0], object.vertex[2][1] );
			glVertex2f( object.vertex[3][0], object.vertex[3][1] );
		glEnd();

		// xNg̕`
		// ̌vZ͂悭ȂDv0v1狁߂ׂD
		double vec[2];
		double dx = object.vertex[0][0] - object.vertex[1][0];
		double dy = object.vertex[0][1] - object.vertex[1][1];
		double d = sqrt( dx*dx + dy*dy );
		vec[0] = d * object.dx + object.px;
		vec[1] = d * object.dy + object.py;
		glBegin(GL_LINES);
			glVertex2f( object.px, object.py);
			glVertex2f( vec[0], vec[1]);
		glEnd();

		GLDrawString( object.px + 0.7*d, object.py, object.name.c_str() );

	End2d();
}
*/